"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CancelFailedBillingBatch = exports.CleanFailedEvaluationPlanData = exports.RemoveEvaluationPlanProcess = exports.CreateQueueL = exports.SetAuditorInteractionsNumbersFromPercentage = exports.StartEvaluationPlan = exports.CreateEvaluationPlan = exports.EvaluationPlanService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var evaluation_plan_model_1 = require("../models/evaluation-plan.model");
var mssql_service_1 = require("../db/mssql.service");
var redis_service_1 = require("../db/redis.service");
var promises_shared_1 = require("../shared/promises.shared");
var bluebird_1 = require("bluebird");
var linguistic_service_1 = require("./linguistic.service");
var billing_batch_implementation_1 = require("../implementation/billing-batch.implementation");
var billing_batch_model_1 = require("../models/billing-batch.model");
var billing_implementation_1 = require("../implementation/billing.implementation");
var interaction_model_1 = require("../models/interaction.model");
var interaction_service_1 = require("./interaction.service");
var billing_model_1 = require("../models/billing.model");
var platform_service_1 = require("./platform.service");
var cognitive_analysis_implementation_1 = require("../implementation/cognitive-analysis.implementation");
var process_service_1 = require("./process.service");
var batch_schedule_service_1 = require("./batch-schedule.service");
var utilities_1 = require("../shared/utilities");
var html_to_pdf = require('html-pdf');
var logger = log4js.getLogger('ServerScripts');
function EvaluationPlanService(app) {
    app.post('/api/evaluationplan/pdf/', function (req, res) {
        var htmlBuffer = new Buffer(req.body.html, 'base64');
        var uriBuffer = new Buffer(req.body.uri, 'base64');
        var html = htmlBuffer.toString("ascii");
        var uri = uriBuffer.toString("ascii");
        var content = "<!doctype html>\n\t\t<html>\n\t\t   <head>\n\t\t\t\t<meta charset=\"utf-8\">\n\t\t\t\t<title>".concat(req.body.id, "</title>\n\t\t\t\t<style>\n\t\t\t\tbody{\n\t\t\t\t\tbox-sizing: border-box;\n\t\t\t\t\tfont-family:arial;\n\t\t\t\t\tpadding:0 10px;\n\t\t\t\t\tfont-size:10px;\n\t\t\t\t  }\n\t\t\t\tdiv.in {\n\t\t\t\t\tbackground-color: #f0f8fb;\n\t\t\t\t\tcolor: #0f4979;\n\t\t\t\t\tborder: 1px solid #8ec8e0;\n\t\t\t\t\tpadding: 15px;\n\t\t\t\t\tmargin-bottom: 15px;\n\t\t\t\t}\n\t\t\t\t\n\t\t\t\tdiv.out{\n\t\t\t\t\tbackground-color: #f8faf5;\n\t\t\t\t\tcolor: #3f5c30;\n\t\t\t\t\tborder: 1px solid #aeca8d;\n\t\t\t\t\tpadding: 15px;\n\t\t\t\t\tmargin-bottom: 15px;\n\t\t\t\t}\n\t\t\t\tli.in[_ngcontent-c6] {\n\t\t\t\t\tbackground-color: #F0F8FB;\n\t\t\t\t\tcolor: #fff;\n\t\t\t\t}\n\t\t\t\t\n\t\t\t\tli.out {\n\t\t\t\t\tbackground-color: #EDF3E5;\n\t\t\t\t\tcolor: #fff;\n\t\t\t\t}\n\t\t\t\t.page-break {\n\t\t\t\t\tposition: relative;\n\t\t\t\t\theight: 100%;\n\t\t\t\t\twidth: 100%;\n\t\t\t\t\tdisplay: block;\n\t\t\t\t\tpage-break-after: always;\n\t\t\t\t\toverflow: hidden;\n\t\t\t\t  }\n\t\t\t\t</style>\n\t\t\t</head>\n\t\t\t<body>\n\t\t\t").concat(html, "\n\t\t\t</body>\n\t\t</html>");
        html_to_pdf.create(content, {
            format: 'A4',
            "header": {
                "height": "20mm",
                "contents": "<div style=\"text-align: center;display:flex\">".concat(uri.length > 60 ? uri.substring(0, 58) : uri, "...</div>")
            }, "footer": {
                "height": "20mm",
                "contents": '<span style="color: #444;">{{page}}</span>/<span>{{pages}}</span>'
            }
        }).toBuffer(function (err, buffer) {
            if (err) {
                res.json({ status: false });
            }
            else {
                res.json({
                    status: true,
                    pdf: buffer.toString("base64")
                });
            }
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan_by_name/:search_name', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_name = req.params.search_name || '';
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Requesting evaluation plan by name.');
            return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, search_name);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan_by_name] Received evaluation plan data.');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Error executing request.');
            logger.error('[EvaluationPlanService::get_evaluationplan_by_name] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplans/:search_criteria/:search_expression?', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans] Received ' + req.method + ' post method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var search_criteria = req.params.search_criteria || '';
        var search_expression = req.params.search_expression || '';
        var disabled = req.query.disabled === 'true' || false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetEvaluationPlans(vcc, search_criteria, search_expression, disabled);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_summary/', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        var isAdmin = req.body.idAgent || false;
        var idAgent = req.body.idAgent || null;
        var idAuditor = req.body.idAuditor || null;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansSummary(vcc, isAdmin, idAgent, idAuditor, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_summary] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_summary] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_auditor/:idAuditor', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var currentUserRole = res.locals.user.role;
        var currentUserId = res.locals.userId;
        var userFuntion = null;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationPlans = req.body.evaluationPlans || [];
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        params.view = req.body.view || null;
        params.efectivityMode = req.body.efectivityMode;
        params.efectivityPlans = req.body.efectivityPlans || [];
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            logger.info('[EvaluationService::get_evaluationplans_auditor] Calling GetEvaluations SP for VCC|IdAuditor: ' + vcc + '|' + currentUserId);
            switch (params.view) {
                case 'AUDITOR_INBOX':
                    userFuntion = 'auditor';
                    break;
                case 'BILLING_DETAIL_MODAL':
                    userFuntion = 'admin';
                    break;
                case 'EVALUATION':
                    userFuntion = 'admin';
                    break;
                default:
                    userFuntion = 'auditor';
                    break;
            }
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Calling SP GetEvaluationPlansAuditor from local DB');
            if (params.efectivityMode) {
                return mssql_service_1.MSSql.GetEfectivityPlansAuditor(vcc, currentUserId, params.campaigns.toString(), params.evaluationPlans.toString(), params.completed, userFuntion, params.efectivityPlans);
            }
            else {
                return mssql_service_1.MSSql.GetEvaluationPlansAuditor(vcc, currentUserId, params.campaigns.toString(), params.limitDate, params.evaluationPlans.toString(), params.evaluationFormats.toString(), params.modalities.toString(), params.completed, userFuntion);
            }
        })
            .then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_auditor] Received result from SP GetEvaluationPlansAuditor from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_auditor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_evaluationplans_agent/:idAgent', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var idAgent = res.locals.userId;
        var params = new evaluation_plan_model_1.EvaluationPlanFilters;
        params.evaluationFormats = req.body.evaluationFormats || [];
        params.campaigns = req.body.campaigns || [];
        params.limitDate = req.body.limitDate || '2050-01-01';
        params.modalities = req.body.modalities || [];
        params.completed = req.body.completed || false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Calling SP GetEvaluationPlansAgent from local DB');
            return mssql_service_1.MSSql.GetEvaluationPlansAgent(vcc, idAgent, params.campaigns.toString(), params.limitDate, params.evaluationFormats.toString(), params.modalities.toString(), params.completed);
        }).then(function (evaluationplans) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplans_agent] Received result from SP GetEvaluationPlansAgent from local DB');
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationplans });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplans_agent] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/create_evaluationplan', function (req, res, next) {
        logger.info("[EvaluationPlanService::create] Getting ".concat(req.method, " method on '").concat(req.path, "'"));
        var vcc = res.locals.vcc;
        var vccType = res.locals.type;
        var userId = res.locals.userId;
        var newEvaluationPlan = (req.body.evaluationPlan || {});
        var newProcess = (req.body.process || null);
        newEvaluationPlan.id = newEvaluationPlan.description.toLowerCase();
        newEvaluationPlan.percentage = newEvaluationPlan.evaluatePercentage / 100;
        newEvaluationPlan.filterByContentPercentage = newEvaluationPlan.evaluatePercentage;
        if (newProcess)
            newEvaluationPlan.processId = newProcess.name;
        logger.info("[EvaluationPlanService::create(".concat(newEvaluationPlan.id, ")] Getting vcc::userId::evaluationplan::process => ").concat(vcc, "::").concat(userId, "::").concat((newProcess === null || newProcess === void 0 ? void 0 : newProcess.name) || null, "::").concat(newEvaluationPlan.id));
        var params = new interaction_model_1.InteractionSearch;
        params = (0, interaction_service_1.GetInteractionSearchFromRequest)(newEvaluationPlan.idEvaluationFormat, newEvaluationPlan.interactionSearch, false);
        params.searchType = 'TO_OPERATE';
        var hasAutomaticPart = (['A', 'X'].includes(newEvaluationPlan.evaluation_mode)
            ||
                (newEvaluationPlan.evaluation_mode === 'M'
                    &&
                        newEvaluationPlan.filterByContentQuery
                    &&
                        newEvaluationPlan.filterByContentQueryName)) ? true : false;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            if (newEvaluationPlan.scheduleData)
                return mssql_service_1.MSSql.GetBatchScheduleById(vcc, newEvaluationPlan.id);
            else
                return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, newEvaluationPlan.id);
        })
            .then(function (result) {
            if (result) {
                if (newEvaluationPlan.scheduleData)
                    throw 'SERVER_ERROR_SCHEDULE_ALREADY_EXISTS';
                else
                    throw 'SERVER_ERROR_EVALUATION_PLAN_ALREADY_EXISTS';
            }
            if (hasAutomaticPart) {
                newProcess.start = false;
                newProcess.type = 'QUALITY';
            }
            if (newEvaluationPlan.scheduleData)
                return CreateScheduleForEvaluationPlan(vcc, vccType, newEvaluationPlan, newProcess, hasAutomaticPart, params, userId);
            else
                return CreateEvaluationPlan(vcc, vccType, newEvaluationPlan, newProcess, hasAutomaticPart, params, userId);
        })
            .then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_FAILED';
            if (res.headersSent)
                return;
            logger.info("[EvaluationPlanService::create(".concat(newEvaluationPlan.id, ")] The EvaluationPlan batch has been finished."));
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error("[EvaluationPlanService::create(".concat(newEvaluationPlan.id, ")] Error on '").concat(req.path, "': ").concat(errorMsg));
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            CancelFailedBillingBatch(vcc, newEvaluationPlan);
            CleanFailedEvaluationPlanData(vcc, newEvaluationPlan);
            RemoveEvaluationPlanProcess(vcc, newEvaluationPlan);
        });
    });
    app.post('/api/evaluationplan/create_evaluationplanX', function (req, res, next) {
        logger.info('[EvaluationPlanService::create_evaluationplan] Getting ' + req.method + ' method on \'' + req.path + '\'');
        var userId = res.locals.userId;
        var vcc = res.locals.vcc;
        var vccType = res.locals.type;
        var processAgentIds = [];
        var nowUtc = moment().utc().toDate();
        var newEvaluationPlan = (req.body || {});
        if (newEvaluationPlan.evaluatePercentage > 100)
            newEvaluationPlan.evaluatePercentage = newEvaluationPlan.evaluatePercentage % 100;
        newEvaluationPlan.evaluatePercentage = newEvaluationPlan.evaluatePercentage / 100;
        var params = new interaction_model_1.InteractionSearch;
        params = (0, interaction_service_1.GetInteractionSearchFromRequest)(req.body.idEvaluationFormat, newEvaluationPlan.interactionSearch, false);
        params.searchType = 'TO_OPERATE';
        newEvaluationPlan.createdDate = nowUtc;
        newEvaluationPlan.createdByUserId = userId;
        newEvaluationPlan.lastModifiedDate = nowUtc;
        newEvaluationPlan.lastModifiedByUserId = userId;
        var channelInteractions = [];
        var totalSyncInteraction = 0;
        var totalAudio = 0;
        var audios = [];
        var totalSyncDetailPhrases = 0;
        var totalSyncDetailRawPhrases = 0;
        var asyncInteractions = [];
        var totalAsyncInteraction = 0;
        var totalAsyncDetail = 0;
        var totalAsyncDetailRawChar = 0;
        var totalAsyncDetailRawPhrases = 0;
        var totalAsyncDetailPhrases = 0;
        var hasAutomaticPart = false;
        if (newEvaluationPlan.evaluation_mode === 'A'
            || newEvaluationPlan.evaluation_mode === 'X'
            || (newEvaluationPlan.evaluation_mode === 'M'
                && newEvaluationPlan.filterByContentQuery
                && newEvaluationPlan.filterByContentQueryName))
            hasAutomaticPart = true;
        var batch = {};
        var billingVccRates = new billing_model_1.BillingVCCRates();
        var cognitiveService;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Checking if exists an EV by name::vcc -> ' + newEvaluationPlan.description + '::' + vcc);
            return mssql_service_1.MSSql.GetEvaluationPlanByName(vcc, newEvaluationPlan.description);
        })
            .then(function (result) {
            if (result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_EXISTS';
            newEvaluationPlan.id = newEvaluationPlan.description;
            logger.info('[EvaluationPlanService::create_evaluationplan] The given name doesn\'t exists, saving the new plan.');
            return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
        })
            .then(function (result) {
            cognitiveService = result;
            if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Searching evaluation format\'s campaigns');
                return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
            }
            else {
                return (0, bluebird_1.resolve)(true);
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Campaigns on evaluationFormat: ' + result.length);
            if (hasAutomaticPart) {
            }
            else {
                if (params.environmentType && (params.environmentType === 'allegro' || params.environmentType === 'i6' || params.environmentType === 'i-ftp' || params.environmentType === 'i-sftp')) {
                    if (params.evaluationFormatId && (params.campaigns && params.campaigns.length === 0)) {
                        if (!result || result.length === 0)
                            throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                        params.campaigns = result.map(function (r) { return r.id; });
                    }
                }
            }
            return Promise.resolve(true);
        }).then(function (result) {
            if (hasAutomaticPart) {
                return Promise.resolve(true);
            }
            else {
                if (params.environmentType
                    &&
                        ['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(params.environmentType)) {
                    return Promise.resolve(true);
                }
                else {
                    var ftpCSVRedisKey = params.evaluationFormatId ? 'FTP_EVALUATION_PLAN:' : 'FTP_PROCESS:';
                    return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey + userId);
                }
            }
        }).then(function (result) {
            if (params.environmentType === 'ftp' || params.environmentType === 'sftp')
                if (result === null)
                    throw 'SERVER_ERROR_CSV_DATA_EXPIRED';
            var promiseArray = [];
            if (hasAutomaticPart) {
                promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsByProcessId(vcc, newEvaluationPlan.processId, 'RAW'));
            }
            else {
                logger.info('[EvaluationPlanService::create_evaluationplan] Requesting not evaluated interactions from HistoricalData');
                promiseArray = (0, interaction_service_1.GetChannelNewInteractionSummaryPromises)(vcc, vccType, false, 'DETAILED', params, result.data);
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Getting result for ' + result.length + ' channels');
            if (!result || result.length === 0)
                throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
            if (hasAutomaticPart) {
                newEvaluationPlan.channelInteractions = result[0];
                if (newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.Channel === 'CALL'; })) {
                    audios = newEvaluationPlan.channelInteractions.filter(function (ci) { return (ci === null || ci === void 0 ? void 0 : ci.Channel) === 'CALL'; });
                    totalSyncInteraction = audios.map(function (item) { return item['InteractionId']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
                    totalAudio = audios.length;
                    totalSyncDetailPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return (ci === null || ci === void 0 ? void 0 : ci.Channel) === 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0) || 0;
                }
                else {
                    audios = [];
                    totalSyncInteraction = 0;
                    totalAudio = 0;
                    totalSyncDetailPhrases = 0;
                }
                if (newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel !== 'CALL'; })) {
                    asyncInteractions = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; });
                    totalAsyncInteraction = asyncInteractions.map(function (item) { return item['id']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
                    totalAsyncDetail = newEvaluationPlan.channelInteractions.filter(function (r) { return r.channel && r.channel !== 'CALL'; }).length;
                    totalAsyncDetailRawChar = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.rawChar : a) + b.rawChar); }, 0);
                    totalAsyncDetailRawPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? Math.ceil(a.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase()) : a) + Math.ceil(b.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase())); }, 0);
                    totalAsyncDetailPhrases = newEvaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0);
                }
                else {
                    asyncInteractions = [];
                    totalAsyncInteraction = 0;
                    totalAsyncDetail = 0;
                    totalAsyncDetailRawChar = 0;
                    totalAsyncDetailRawPhrases = 0;
                    totalAsyncDetailPhrases = 0;
                }
                var billing = new billing_implementation_1.BillingImplementation();
                return billing.GetVCCRates(vcc, newEvaluationPlan.processId);
            }
            else {
                channelInteractions = result;
                newEvaluationPlan.channelInteractions = [];
                var totalInteractionsToSave_1 = 0;
                channelInteractions.map(function (ci) { return totalInteractionsToSave_1 += ci && ci.totalInteractions ? ci.totalInteractions : 0; });
                totalInteractionsToSave_1 = Math.round(totalInteractionsToSave_1 * newEvaluationPlan.evaluatePercentage);
                channelInteractions.map(function (ci) {
                    if (ci && ci.totalInteractions) {
                        if (ci.interactionType === 'CALL') {
                            var iv = {};
                            iv = ci;
                            var idTitle = 'InteractionId';
                            var sampleResult = (0, linguistic_service_1.GetInteractionsSample)(iv.interactions, newEvaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_1);
                            totalSyncInteraction = sampleResult.sampleInteractionIds.length;
                            audios = sampleResult.sampleInteractionDetails;
                            totalAudio = audios.length;
                            totalInteractionsToSave_1 -= sampleResult.sampleInteractionIds.length;
                            newEvaluationPlan.channelInteractions = newEvaluationPlan.channelInteractions.concat(audios);
                            audios.map(function (a) {
                                if (a.Agent)
                                    processAgentIds.push(a.Agent);
                            });
                        }
                        else {
                            var aiv = {};
                            aiv = ci;
                            asyncInteractions = [];
                            var idTitle = 'id';
                            var sampleResult = (0, linguistic_service_1.GetInteractionsSample)(aiv.interactions, newEvaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_1);
                            totalAsyncInteraction += sampleResult.sampleInteractionIds.length;
                            asyncInteractions = sampleResult.sampleInteractionDetails;
                            totalAsyncDetail += asyncInteractions.length;
                            totalInteractionsToSave_1 -= sampleResult.sampleInteractionIds.length;
                            newEvaluationPlan.channelInteractions = newEvaluationPlan.channelInteractions.concat(asyncInteractions);
                            aiv.interactions.map(function (a) {
                                if (a.firstTakedAgent)
                                    processAgentIds.push(a.firstTakedAgent);
                                if (a.lastTakedAgent)
                                    processAgentIds.push(a.lastTakedAgent);
                            });
                        }
                    }
                });
                return true;
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Receiving rates data');
            if (result === null)
                throw 'SERVER_ERROR_RATES_NOT_FOUND';
            var promiseArray = [];
            if (hasAutomaticPart) {
                billingVccRates.SetBillingVCCRates(result);
                batch.vcc = vcc;
                batch.totalInteraction = totalSyncInteraction + totalAsyncInteraction;
                batch.totalAsync = totalAsyncInteraction;
                batch.totalAsyncDetail = totalAsyncDetail;
                batch.totalSync = totalSyncInteraction;
                batch.totalAudio = totalAudio;
                batch.durationTime = 0;
                batch.roundedTime = 0;
                audios.map(function (a) {
                    batch.durationTime += a.DurationTime;
                    batch.roundedTime += Math.ceil(a.DurationTime / 60) * 60;
                });
                batch.totalQueries = 0;
                batch.totalModels = 0;
                batch.batchTypeName = 'EVALUATION_PLAN';
                batch.batchTypeId = newEvaluationPlan.id;
                batch.templateId = newEvaluationPlan.templateId;
                batch.processId = newEvaluationPlan.processId;
                batch.userApi = userId;
                batch.startDate = nowUtc;
                promiseArray.push((0, platform_service_1.GetPlatformVersion)());
                if (batch.templateId) {
                    promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(batch.vcc, batch.templateId, null));
                    promiseArray.push(GetEvaluationPlanModels(batch.vcc, batch.templateId));
                }
                else {
                    promiseArray.push(Promise.resolve([]));
                    promiseArray.push(Promise.resolve([]));
                }
                promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(batch.vcc, newEvaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
                promiseArray.push(mssql_service_1.MSSql.GetAllegroBillingInteractionsForEvaluationPlan(batch.vcc, newEvaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
            }
            else {
                if (vccType !== 'OTHER')
                    promiseArray.push((0, linguistic_service_1.ImportAgents)(vcc, vccType, processAgentIds));
                else
                    promiseArray.push(Promise.resolve(true));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Receiving extra data for batch');
            if (hasAutomaticPart) {
                var alreadyTranscribedAudio = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedAudio || 0;
                var alreadyDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedDurationTime || 0;
                var alreadyRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedRoundedDurationTime || 0;
                var alreadyTranscribedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidDurationTime || 0;
                var alreadyTranscribedRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidRoundedDurationTime || 0;
                totalSyncDetailRawPhrases = (batch.durationTime - alreadyDurationTime) / billingVccRates.cognitiveAnalysisAverageSecondsPerRequest;
                var estimatedRequests = {
                    modelRequestsFromCallTime: totalSyncDetailRawPhrases,
                    modelRequestFromCallAlreadyPhrases: totalSyncDetailPhrases,
                    modelRequestsFromTextRawChar: totalAsyncDetailRawPhrases,
                    modelRequestsFromTextAlreadyPhrases: totalAsyncDetailPhrases
                };
                if (result[0] === null)
                    throw 'INSTALLATIONID_MISSING';
                batch.totalCallPhrases = totalSyncDetailRawPhrases || 0;
                batch.alreadyCallPhrases = totalSyncDetailPhrases || 0;
                batch.totalTextPhrases = totalAsyncDetailRawPhrases || 0;
                batch.alreadyTextPhrases = totalAsyncDetailPhrases || 0;
                batch.startDate = nowUtc;
                batch.installationId = result[0].installationId;
                batch.totalQueries = (newEvaluationPlan.evaluation_mode === 'A' || newEvaluationPlan.evaluation_mode === 'X') ? result[1].length : 0;
                batch.totalModels = result[2].length;
                batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromNone(estimatedRequests);
                batch.alreadyTranscribedAudio = alreadyTranscribedAudio ? alreadyTranscribedAudio : 0;
                batch.alreadyDurationTime = alreadyDurationTime ? alreadyDurationTime : 0;
                batch.alreadyRoundedDurationTime = alreadyRoundedDurationTime ? alreadyRoundedDurationTime : 0;
                batch.alreadyTranscribedDurationTime = alreadyTranscribedDurationTime ? alreadyTranscribedDurationTime : 0;
                batch.alreadyTranscribedRoundedDurationTime = alreadyTranscribedRoundedDurationTime ? alreadyTranscribedRoundedDurationTime : 0;
                batch.inProcessAudio = result[4].inProcessAudio ? result[4].inProcessAudio : 0;
                batch.inProcessDurationTime = result[4].inProcessDurationTime ? result[4].inProcessDurationTime : 0;
                if (newEvaluationPlan.filterByContent
                    && newEvaluationPlan.filterByContentQuery
                    && newEvaluationPlan.filterByContentQueryName)
                    batch.totalQueries++;
                var billing = new billing_batch_implementation_1.BillingBatchImplementation();
                return billing.SaveBatchSummary(batch);
            }
            else {
                return { status: true };
            }
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Batch was saved.');
            if (result && result.status === false)
                throw result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
            return SaveEvaluationPlan(vcc, newEvaluationPlan);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_CREATE_EVALUATIONPLAN_FAILED';
            logger.info('[EvaluationPlanService::create_evaluationplan] The EV was saved on EvaluationPlan table.');
            return mssql_service_1.MSSql.BulkInteractionEvaluationPlan(vcc, newEvaluationPlan);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_SAVE_INTERACTION_EVALUATIONPLAN_FAILED';
            logger.info('[EvaluationPlanService::create_evaluationplan] getting result: ' + result + ' after BulkInteractionEvaluationPlan');
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, newEvaluationPlan.id));
            if (newEvaluationPlan.filterByContentQuery
                && newEvaluationPlan.filterByContentQueryName) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Adding into evaluationPlan queue.');
                promiseArray.push(CreateQueueL(vcc, newEvaluationPlan.processId, 'evaluationPlan', newEvaluationPlan.id));
            }
            if (['A', 'X'].includes(newEvaluationPlan.evaluation_mode)) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Adding into process queue');
                promiseArray.push(CreateQueueL(vcc, newEvaluationPlan.processId, 'ability', newEvaluationPlan.id));
                promiseArray.push(CreateQueueL(vcc, newEvaluationPlan.processId, 'tipification', newEvaluationPlan.id));
            }
            if (hasAutomaticPart) {
                promiseArray.push((0, linguistic_service_1.BeginProcess)(vcc, newEvaluationPlan.processId));
            }
            return Promise.all(promiseArray);
        }).then(function (result) {
            logger.info('[EvaluationPlanService::create_evaluationplan] Calling function to save interactions per auditor.');
            newEvaluationPlan.uniqueInteractions = [];
            newEvaluationPlan.uniqueInteractions = result[0].reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    if (current.dispositionCode) {
                        var arrDispositionCode = [];
                        arrDispositionCode = x.dispositionCode ? x.dispositionCode.split(',') : [];
                        if (!arrDispositionCode.includes(current.dispositionCode))
                            arrDispositionCode.push(current.dispositionCode);
                        acc.find(function (item) { return item.id === current.id; })['dispositionCode'] = arrDispositionCode.join(',');
                    }
                    return acc;
                }
            }, []);
            logger.info('[EvaluationPlanService::create_evaluationplan] UniqueInteractionIds: ' + newEvaluationPlan.uniqueInteractions.length);
            logger.info('[EvaluationPlanService::create_evaluationplan] UniqueInteractionIds: ' + (totalSyncInteraction + totalAsyncInteraction));
            logger.info('[EvaluationPlanService::create_evaluationplan] evaluation_mode::filterByContentQuery::filterByContentQueryName => ', newEvaluationPlan.evaluation_mode, '::', newEvaluationPlan.filterByContentQuery, '::', newEvaluationPlan.filterByContentQueryName);
            if ((newEvaluationPlan.evaluation_mode === 'A'
                ||
                    newEvaluationPlan.evaluation_mode === 'X')
                && newEvaluationPlan.filterByContentQuery === null
                && newEvaluationPlan.filterByContentQueryName === null) {
                logger.info('[EvaluationPlanService::create_evaluationplan] Asignando evaluaciones para auditor SPEECH');
                var auditor = {
                    idAuditor: 'SPEECH',
                    numberInteractions: newEvaluationPlan.uniqueInteractions.length
                };
                if (newEvaluationPlan.evaluation_mode === 'A')
                    newEvaluationPlan.auditors = [];
                newEvaluationPlan.auditors.push(auditor);
            }
            return SaveAuditorEP(vcc, newEvaluationPlan);
        }).then(function (resultSave) {
            if (resultSave === null)
                throw 'SERVER_ERROR_SAVE_AUDITOREP_FAILED';
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::create_evaluationplan] The EV save proccess has been finished.');
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::create_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
            CancelFailedBillingBatch(vcc, newEvaluationPlan);
            CleanFailedEvaluationPlanData(vcc, newEvaluationPlan);
            RemoveEvaluationPlanProcess(vcc, newEvaluationPlan);
        });
    });
    app.post('/api/evaluationplan/save_evaluationplan', function (req, res, next) {
        logger.info('[EvaluationPlanService::save_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var evaluationPlan = (req.body || {});
        evaluationPlan.createdDate = null;
        evaluationPlan.createdByUserId = null;
        evaluationPlan.lastModifiedDate = nowUtc;
        evaluationPlan.lastModifiedByUserId = userId;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            SaveEvaluationPlan(vcc, evaluationPlan);
        }).then(function (result) {
            return mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(function (result) {
            evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage % 100;
            if (evaluationPlan.evaluatePercentage > 1)
                evaluationPlan.evaluatePercentage = evaluationPlan.evaluatePercentage / 100;
            if (evaluationPlan.evaluatePercentage < 1 && evaluationPlan.evaluatePercentage > 0) {
                return mssql_service_1.MSSql.SaveInteractionEvaluationPlanSample(vcc, evaluationPlan.id, evaluationPlan.evaluatePercentage);
            }
            else {
                return true;
            }
        }).then(function (result) {
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlan.id);
        })
            .then(function (result) {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE';
            evaluationPlan.interactions = result;
            evaluationPlan.uniqueInteractions = [];
            evaluationPlan.uniqueInteractions = result.reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            return SaveAuditorEP(vcc, evaluationPlan);
        }).then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_SAVE'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[EvaluationPlanService::save_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluationplan/:evaluationplan_id', function (req, res, next) {
        logger.info('[EvaluationPlanService::get_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlan = {};
        var auditorEvaluationPlan = [];
        var evaluationPlanId = req.params.evaluationplan_id;
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationPlanId) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan ID is null');
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Sending request to SP GetEvaluationPlan for EP ID: ' + evaluationPlanId);
            return mssql_service_1.MSSql.GetEvaluationPlan(vcc, evaluationPlanId);
        }).then(function (result) {
            if (!result) {
                logger.error('[EvaluationPlanService::get_evaluationplan] Evaluation plan doesn\'t exists on DB');
                throw 'EVALUATION_PLAN_SERVER_ERROR_CANNOT_GET';
            }
            logger.info('[EvaluationPlanService::get_evaluationplan] Received evaluation plan from DB');
            evaluationPlan = result || evaluationPlan;
            return mssql_service_1.MSSql.GetAuditorsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received auditors from EP: ' + result.length);
            evaluationPlan.auditors = result || evaluationPlan.auditors;
            return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            logger.info('[EvaluationPlanService::get_evaluationplan] Received interaction details(audios/asyncinteractions): ' + result.length);
            evaluationPlan.interactions = result.reduce(function (acc, current) {
                var x = acc.find(function (item) { return item.id === current.id; });
                if (!x) {
                    return acc.concat([current]);
                }
                else {
                    return acc;
                }
            }, []);
            evaluationPlan.interactions = evaluationPlan.interactions || [];
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: evaluationPlan });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::get_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplan', function (req, res, next) {
        logger.info('[EvaluationPlanService::delete_evaluationplan] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var evaluationPlan = (req.body || {});
        var evaluationPlanId = evaluationPlan.id;
        logger.info("[EvaluationPlanService::delete_evaluationplan] Deleting plan ".concat(evaluationPlanId, " by the user ").concat(userId));
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationPlan)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_ID';
            return mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlanId);
        })
            .then(function (result) {
            if (!result)
                throw 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE';
            return Promise.resolve(true);
        })
            .then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: result, description: (result ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: result });
        })
            .catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
            logger.error('[EvaluationPlanService::delete_evaluationplan] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/delete_evaluationplans', function (req, res, next) {
        logger.info('Se recibió ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var evaluationPlans = req.body.evaluationPlans || [];
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            evaluationPlans.forEach(function (evaluationPlan) {
                promiseArray.push(mssql_service_1.MSSql.DeleteAuditorEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(function (result) {
            if (!evaluationPlans)
                throw 'EVALUATION_PLAN_SERVER_ERROR_MISSING_IDS';
            var promiseArray = [];
            evaluationPlans.forEach(function (evaluationPlan) {
                promiseArray.push(mssql_service_1.MSSql.DeleteEvaluationPlan(vcc, evaluationPlan.id));
            });
            return Promise.all(promiseArray);
        }).then(function (resultDeletes) {
            var result;
            if (resultDeletes.filter(function (result) { return (result !== true); }).length > 0) {
                result = false;
                logger.error('No se pudo eliminar todos los elementos solicitados.');
            }
            else {
                result = true;
                logger.info('Evaluation plans eliminados correctamente');
            }
            return result;
        }).then(function (resultDeletes) {
            if (res.headersSent)
                return;
            res.json({ status: resultDeletes, description: (resultDeletes ? 'SERVER_SUCCESS_MESSAGE' : 'EVALUATION_PLAN_SERVER_ERROR_ON_DELETE'), data: resultDeletes });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/evaluationplan/get_evaluation_items/:evaluationplan_id', function (req, res, next) {
        logger.info("[EvaluationPlanService::get_groups] Received ".concat(req.method, " on ").concat(req.path, " for vcc ").concat(res.locals.vcc));
        var vcc = res.locals.vcc;
        var evaluationPlanId = req.params.evaluationplan_id || null;
        (0, promises_shared_1.DummyPromise)()
            .then(function (_) {
            var promiseArray = [];
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanGroups(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanSubGroups(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilities(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanScores(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueries(vcc, evaluationPlanId));
            promiseArray.push(mssql_service_1.MSSql.GetEvaluationPlanAbilitiesQueriesEntities(vcc, evaluationPlanId));
            return Promise.all(promiseArray);
        })
            .then(function (result) {
            var _a, _b;
            if (res.headersSent)
                return;
            var resultObject = {
                groups: result[0],
                subgroups: result[1],
                abilities: result[2]
            };
            if ((_a = resultObject.abilities) === null || _a === void 0 ? void 0 : _a.length) {
                if ((_b = result[4]) === null || _b === void 0 ? void 0 : _b.length) {
                    result[4].forEach(function (aq) {
                        var _a;
                        aq.entities = ((_a = result[5]) === null || _a === void 0 ? void 0 : _a.length) ? result[5].filter(function (aqe) { return aqe.abilityQueryId === aq.id; }) : [];
                    });
                }
                resultObject.abilities.forEach(function (a) {
                    var _a, _b;
                    a.scores = ((_a = result[3]) === null || _a === void 0 ? void 0 : _a.length) ? result[3].filter(function (s) { return s.idAbility === a.id; }) : [];
                    a.queries = ((_b = result[4]) === null || _b === void 0 ? void 0 : _b.length) ? result[4].filter(function (aq) { return aq.abilityId === a.id; }) : [];
                });
            }
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultObject });
        })
            .catch(function (err) {
            var groupMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.info("[EvaluationPlanService::get_groups] Error on ".concat(req.path, ": ").concat(errorMsg));
            if (res.headersSent)
                return;
            res.json({ status: false, description: groupMsg, error: errorMsg });
        });
    });
    app.post('/api/evaluationplan/get_filters_from_evaluation', function (req, res, next) {
        logger.info('[EvaluationPlanService::evaluation] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.instance;
        var type = res.locals.type;
        var userId = res.locals.userId;
        var referrer = res.locals.pageReferrer.split('/').pop();
        var searchType = null;
        var campaigns = req.body.campaigns ? req.body.campaigns : [];
        var efectivityMode = req.body.efectivityMode ? true : false;
        switch (referrer) {
            case 'auditorinbox':
                searchType = 'PENDING';
                break;
            case 'evaluation':
                searchType = 'HISTORY';
                break;
            case 'evaluation-agent':
                searchType = 'HISTORY';
                break;
            case 'evaluation-admin':
                searchType = 'HISTORY';
                break;
        }
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetFiltersFromEvaluation(vcc, userId, campaigns, searchType, efectivityMode);
        }).then(function (result) {
            if (!result)
                throw 'SERVER_ERROR_AGENTS_SEARCH_FAILED';
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[AgentsService::evaluation] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
}
exports.EvaluationPlanService = EvaluationPlanService;
function CreateEvaluationPlan(vcc, vccType, evaluationPlan, process, hasAutomaticPart, params, userId) {
    var _a;
    var hasCampaigns = ((_a = params === null || params === void 0 ? void 0 : params.campaigns) === null || _a === void 0 ? void 0 : _a.length) ? params.campaigns.length : 0;
    var interactionsSummary = new evaluation_plan_model_1.EvaluationPlanInteractionsSummary();
    var wasEvaluationPlanCreated = false;
    evaluationPlan.status = 1;
    evaluationPlan.createdByUserId = userId;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!hasCampaigns) {
            logger.info("[EvaluationPlanService::CreateEvaluationPlan(".concat(evaluationPlan.id, ")] Searching evaluation format's campaigns"));
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
        }
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (hasAutomaticPart) {
        }
        else {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(params.environmentType)) {
                if (params.evaluationFormatId && !hasCampaigns) {
                    if (!result || !result.length)
                        throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                    logger.info("[EvaluationPlanService::CreateEvaluationPlan(".concat(evaluationPlan.id, ")] Campaigns on evaluationFormat: ").concat(result.length));
                    params.campaigns = result.map(function (r) { return r.id; });
                }
            }
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        if (hasAutomaticPart) {
            return (0, process_service_1.CreateProcess)(vcc, vccType, process, params, userId);
        }
        else
            return Promise.resolve({ status: true });
    })
        .then(function (result) {
        if (!result || (result && !result.status))
            throw 'SERVER_ERROR_CREATE_PROCESS_FOR_EVALUATION_PLAN_FAILED';
        return GetInteractionsDataForEvaluationPlan(vcc, vccType, evaluationPlan, hasAutomaticPart, params, userId);
    })
        .then(function (result) {
        interactionsSummary = result;
        if (!hasAutomaticPart && vccType !== 'OTHER')
            return (0, linguistic_service_1.ImportAgents)(vcc, vccType, interactionsSummary.agentIds);
        return Promise.resolve(true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
        logger.info("[EvaluationPlanService::CreateEvaluationPlan(".concat(evaluationPlan.id, ")] Batch was saved."));
        return SaveEvaluationPlan(vcc, evaluationPlan);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_CREATE_EVALUATIONPLAN_FAILED';
        wasEvaluationPlanCreated = true;
        logger.info("[EvaluationPlanService::CreateEvaluationPlan(".concat(evaluationPlan.id, ")] The EvaluationPlan was saved."));
        return mssql_service_1.MSSql.BulkInteractionEvaluationPlan(vcc, evaluationPlan);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_SAVE_INTERACTION_EVALUATIONPLAN_FAILED';
        logger.info("[EvaluationPlanService::CreateEvaluationPlan(".concat(evaluationPlan.id, ")] getting result: ") + result + ' after BulkInteractionEvaluationPlan');
        return mssql_service_1.MSSql.GetInteractionsEvaluationPlan(vcc, evaluationPlan.id);
    })
        .then(function (result) {
        logger.info("[EvaluationPlanService::CreateEvaluationplan(".concat(evaluationPlan.id, ")] Calling function to save interactions per auditor."), result);
        evaluationPlan.uniqueInteractions = [];
        evaluationPlan.uniqueInteractions = result.reduce(function (acc, current) {
            var x = acc.find(function (item) { return item.id === current.id; });
            if (!x) {
                return acc.concat([current]);
            }
            else {
                if (current.dispositionCode) {
                    var arrDispositionCode = [];
                    arrDispositionCode = x.dispositionCode ? x.dispositionCode.split(',') : [];
                    if (!arrDispositionCode.includes(current.dispositionCode))
                        arrDispositionCode.push(current.dispositionCode);
                    acc.find(function (item) { return item.id === current.id; })['dispositionCode'] = arrDispositionCode.join(',');
                }
                return acc;
            }
        }, []);
        logger.info("[EvaluationPlanService::CreateEvaluationplan] UniqueInteractionIds: ".concat(evaluationPlan.uniqueInteractions.length, " (CALL: ").concat(interactionsSummary.totalSyncInteraction, ", TEXT: ").concat(interactionsSummary.totalAsyncInteraction, ")"));
        logger.info("[EvaluationPlanService::CreateEvaluationplan] evaluation_mode::filterByContentQuery::filterByContentQueryName => ".concat(evaluationPlan.evaluation_mode, "::").concat(evaluationPlan.filterByContentQuery, "::").concat(evaluationPlan.filterByContentQueryName, "}"));
        if (['A', 'X'].includes(evaluationPlan.evaluation_mode)
            &&
                evaluationPlan.filterByContentQuery === null
            &&
                evaluationPlan.filterByContentQueryName === null) {
            logger.info('[EvaluationPlanService::CreateEvaluationplan] Saving evaluations for SPEECH auditor');
            var auditor = {
                idAuditor: 'SPEECH',
                numberInteractions: evaluationPlan.uniqueInteractions.length
            };
            if (evaluationPlan.evaluation_mode === 'A')
                evaluationPlan.auditors = [];
            evaluationPlan.auditors.push(auditor);
        }
        return SaveAuditorEP(vcc, evaluationPlan);
    })
        .then(function (result) {
        if (result === null)
            throw 'SERVER_ERROR_SAVE_AUDITOREP_FAILED';
        logger.info("[EvaluationPlanService::CreateEvaluationplan(".concat(evaluationPlan.id, ")] The EV save proccess has been finished."));
        if (hasAutomaticPart && evaluationPlan.evaluationStarted)
            return SaveEvaluationPlanBatch(vcc, evaluationPlan, interactionsSummary, userId);
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_COULD_NOT_SAVE_BATCH';
        return Promise.resolve(result);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error("[EvaluationPlanService::CreateEvaluationplan(".concat(evaluationPlan.id, ")] Error: ") + errorMsg);
        if (wasEvaluationPlanCreated)
            CleanFailedEvaluationPlanData(vcc, evaluationPlan);
        RemoveEvaluationPlanProcess(vcc, evaluationPlan);
        return Promise.reject(err);
    });
}
exports.CreateEvaluationPlan = CreateEvaluationPlan;
function GetInteractionsDataForEvaluationPlan(vcc, vccType, evaluationPlan, hasAutomaticPart, params, userId) {
    var _a;
    var cognitiveService = null;
    var summary = new evaluation_plan_model_1.EvaluationPlanInteractionsSummary();
    (_a = evaluationPlan.channelInteractions) !== null && _a !== void 0 ? _a : (evaluationPlan.channelInteractions = []);
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        return (0, cognitive_analysis_implementation_1.GetCognitiveService)(vcc, null);
    })
        .then(function (result) {
        cognitiveService = result;
        if (hasAutomaticPart)
            return mssql_service_1.MSSql.GetProcessInteractionsByProcessId(vcc, evaluationPlan.processId, 'RAW');
        else {
            if (['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(params.environmentType))
                return Promise.resolve(null);
            else {
                var ftpCSVRedisKey = void 0;
                ftpCSVRedisKey = "FTP_EVALUATION_PLAN:".concat(userId);
                return redis_service_1.Redis.GetCsvProcess(vcc, ftpCSVRedisKey);
            }
        }
    })
        .then(function (result) {
        var promiseArray = [];
        if (hasAutomaticPart) {
            promiseArray.push(Promise.resolve(result));
        }
        else {
            if (['ftp', 'sftp'].includes(params.environmentType) && !result)
                'SERVER_ERROR_CSV_DATA_EXPIRED';
            logger.info('[EvaluationPlanService::CreateEvaluationPlan] Requesting not evaluated interactions from HistoricalData');
            promiseArray = (0, interaction_service_1.GetChannelNewInteractionSummaryPromises)(vcc, vccType, false, 'DETAILED', params, (result === null || result === void 0 ? void 0 : result.data) ? result.data : null);
        }
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        logger.info("[EvaluationPlanService::CreateEvaluationPlan] Getting result for ".concat(result.length, " channels"));
        if (!result || result.length === 0)
            throw 'SERVER_ERROR_NO_INTEGRATED_DATA_FOUND';
        if (hasAutomaticPart) {
            evaluationPlan.channelInteractions = result[0];
        }
        else {
            var channelInteractions = result;
            var totalInteractionsToSave_2 = 0;
            channelInteractions.map(function (ci) { return totalInteractionsToSave_2 += ci && ci.totalInteractions ? ci.totalInteractions : 0; });
            totalInteractionsToSave_2 = Math.round(totalInteractionsToSave_2 * evaluationPlan.evaluatePercentage);
            channelInteractions.map(function (ci) {
                if (ci === null || ci === void 0 ? void 0 : ci.totalInteractions) {
                    var idTitle = null;
                    var sampleResult = null;
                    if (ci.interactionType === 'CALL') {
                        var iv = {};
                        iv = ci;
                        idTitle = 'InteractionId';
                        sampleResult = (0, linguistic_service_1.GetInteractionsSample)(iv.interactions, evaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_2);
                        summary.audios = sampleResult.sampleInteractionDetails;
                        evaluationPlan.channelInteractions = evaluationPlan.channelInteractions.concat(summary.audios);
                        summary.audios.map(function (a) {
                            if (a.Agent)
                                summary.agentIds.push(a.Agent);
                        });
                    }
                    else {
                        var aiv = {};
                        aiv = ci;
                        idTitle = 'id';
                        sampleResult = (0, linguistic_service_1.GetInteractionsSample)(aiv.interactions, evaluationPlan.evaluatePercentage, idTitle, totalInteractionsToSave_2);
                        summary.asyncInteractions = sampleResult.sampleInteractionDetails;
                        evaluationPlan.channelInteractions = evaluationPlan.channelInteractions.concat(summary.asyncInteractions);
                        aiv.interactions.map(function (a) {
                            if (a.firstTakedAgent)
                                summary.agentIds.push(a.firstTakedAgent);
                            if (a.lastTakedAgent)
                                summary.agentIds.push(a.lastTakedAgent);
                        });
                    }
                    totalInteractionsToSave_2 -= sampleResult.sampleInteractionIds.length;
                }
            });
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        if (evaluationPlan.channelInteractions.filter(function (ci) { return ci.Channel === 'CALL'; })) {
            summary.audios = evaluationPlan.channelInteractions.filter(function (ci) { return (ci === null || ci === void 0 ? void 0 : ci.Channel) === 'CALL'; });
            summary.totalSyncInteraction = summary.audios.map(function (item) { return item['InteractionId']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
            summary.totalAudio = summary.audios.length;
            summary.totalSyncDetailPhrases = evaluationPlan.channelInteractions.filter(function (ci) { return (ci === null || ci === void 0 ? void 0 : ci.Channel) === 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0) || 0;
        }
        else {
            summary.audios = [];
            summary.totalSyncInteraction = 0;
            summary.totalAudio = 0;
            summary.totalSyncDetailPhrases = 0;
        }
        if (evaluationPlan.channelInteractions.filter(function (ci) { return ci.channel !== 'CALL'; })) {
            summary.asyncInteractions = evaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; });
            summary.totalAsyncInteraction = summary.asyncInteractions.map(function (item) { return item['id']; }).filter(function (value, index, self) { return self.indexOf(value) === index; }).length;
            summary.totalAsyncDetail = evaluationPlan.channelInteractions.filter(function (r) { return r.channel && r.channel !== 'CALL'; }).length;
            summary.totalAsyncDetailRawChar = evaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.rawChar : a) + b.rawChar); }, 0);
            summary.totalAsyncDetailRawPhrases = evaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? Math.ceil(a.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase()) : a) + Math.ceil(b.rawChar / cognitiveService.GetCognitiveEngineCharsPerPhrase())); }, 0);
            summary.totalAsyncDetailPhrases = evaluationPlan.channelInteractions.filter(function (ci) { return ci.channel && ci.channel !== 'CALL'; }).reduce(function (a, b) { return ((isNaN(a) ? a.totalPhrases : a) + b.totalPhrases); }, 0);
        }
        else {
            summary.asyncInteractions = [];
            summary.totalAsyncInteraction = 0;
            summary.totalAsyncDetail = 0;
            summary.totalAsyncDetailRawChar = 0;
            summary.totalAsyncDetailRawPhrases = 0;
            summary.totalAsyncDetailPhrases = 0;
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        return Promise.resolve(summary);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationPlanService::GetInteractionsDataForEvaluationPlan] Error: ' + errorMsg);
        return Promise.reject(err);
    });
}
function SaveEvaluationPlanBatch(vcc, evaluationPlan, summary, userId) {
    var batch = {};
    var billingVccRates = new billing_model_1.BillingVCCRates();
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!summary)
            return GetInteractionsDataForEvaluationPlan(vcc, null, evaluationPlan, true, null, userId);
        else
            return Promise.resolve(summary);
    })
        .then(function (result) {
        summary !== null && summary !== void 0 ? summary : (summary = result);
        var billing = new billing_implementation_1.BillingImplementation();
        return billing.GetVCCRates(vcc, evaluationPlan.processId);
    })
        .then(function (result) {
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Receiving rates data');
        if (result === null)
            throw 'SERVER_ERROR_RATES_NOT_FOUND';
        var promiseArray = [];
        billingVccRates.SetBillingVCCRates(result);
        batch.vcc = vcc;
        batch.totalInteraction = summary.GetTotalInteractions();
        batch.totalAsync = summary.totalAsyncInteraction;
        batch.totalAsyncDetail = summary.totalAsyncDetail;
        batch.totalSync = summary.totalSyncInteraction;
        batch.totalAudio = summary.totalAudio;
        batch.durationTime = 0;
        batch.roundedTime = 0;
        summary.audios.forEach(function (a) {
            batch.durationTime += a.DurationTime;
            batch.roundedTime += Math.ceil(a.DurationTime / 60) * 60;
        });
        batch.totalQueries = 0;
        batch.totalModels = 0;
        batch.batchTypeName = 'EVALUATION_PLAN';
        batch.batchTypeId = evaluationPlan.id;
        batch.templateId = evaluationPlan.templateId;
        batch.processId = evaluationPlan.processId;
        batch.batchScheduleId = evaluationPlan.batchScheduleId;
        batch.userApi = userId;
        batch.startDate = moment().utc().toDate();
        promiseArray.push((0, platform_service_1.GetPlatformVersion)());
        if (batch.templateId) {
            promiseArray.push(mssql_service_1.MSSql.GetTemplateThemeItems(batch.vcc, batch.templateId, null));
            promiseArray.push(GetEvaluationPlanModels(batch.vcc, batch.templateId));
        }
        else {
            promiseArray.push(Promise.resolve([]));
            promiseArray.push(Promise.resolve([]));
        }
        promiseArray.push(mssql_service_1.MSSql.GetProcessInteractionsHistory(batch.vcc, evaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        promiseArray.push(mssql_service_1.MSSql.GetAllegroBillingInteractionsForEvaluationPlan(batch.vcc, evaluationPlan.processId, billingVccRates.cognitiveAnalysisAverageSecondsPerRequest));
        return Promise.all(promiseArray);
    })
        .then(function (result) {
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Receiving extra data for batch');
        var alreadyTranscribedAudio = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedAudio || 0;
        var alreadyDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedDurationTime || 0;
        var alreadyRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedRoundedDurationTime || 0;
        var alreadyTranscribedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidDurationTime || 0;
        var alreadyTranscribedRoundedDurationTime = result[3].find(function (r) { return r.interactionType === 'CALL'; }).alreadyTranscribedValidRoundedDurationTime || 0;
        summary.totalSyncDetailRawPhrases = (batch.durationTime - alreadyDurationTime) / billingVccRates.cognitiveAnalysisAverageSecondsPerRequest;
        var estimatedRequests = {
            modelRequestsFromCallTime: summary.totalSyncDetailRawPhrases,
            modelRequestFromCallAlreadyPhrases: summary.totalSyncDetailPhrases,
            modelRequestsFromTextRawChar: summary.totalAsyncDetailRawPhrases,
            modelRequestsFromTextAlreadyPhrases: summary.totalAsyncDetailPhrases
        };
        if (result[0] === null)
            throw 'INSTALLATIONID_MISSING';
        batch.totalCallPhrases = summary.totalSyncDetailRawPhrases || 0;
        batch.alreadyCallPhrases = summary.totalSyncDetailPhrases || 0;
        batch.totalTextPhrases = summary.totalAsyncDetailRawPhrases || 0;
        batch.alreadyTextPhrases = summary.totalAsyncDetailPhrases || 0;
        batch.startDate = moment().utc().toDate();
        batch.installationId = result[0].installationId;
        batch.totalQueries = ['A', 'X'].includes(evaluationPlan.evaluation_mode) ? result[1].length : 0;
        batch.totalModels = result[2].length;
        batch.modelRequests = billingVccRates.GetCognitiveAnalysisTotalAveragePhrasesFromNone(estimatedRequests);
        batch.alreadyTranscribedAudio = alreadyTranscribedAudio ? alreadyTranscribedAudio : 0;
        batch.alreadyDurationTime = alreadyDurationTime ? alreadyDurationTime : 0;
        batch.alreadyRoundedDurationTime = alreadyRoundedDurationTime ? alreadyRoundedDurationTime : 0;
        batch.alreadyTranscribedDurationTime = alreadyTranscribedDurationTime ? alreadyTranscribedDurationTime : 0;
        batch.alreadyTranscribedRoundedDurationTime = alreadyTranscribedRoundedDurationTime ? alreadyTranscribedRoundedDurationTime : 0;
        batch.inProcessAudio = result[4].inProcessAudio ? result[4].inProcessAudio : 0;
        batch.inProcessDurationTime = result[4].inProcessDurationTime ? result[4].inProcessDurationTime : 0;
        if (evaluationPlan.filterByContent
            &&
                evaluationPlan.filterByContentQuery
            &&
                evaluationPlan.filterByContentQueryName)
            batch.totalQueries++;
        var billing = new billing_batch_implementation_1.BillingBatchImplementation();
        return billing.SaveBatchSummary(batch);
    })
        .then(function (result) {
        if ((result === null || result === void 0 ? void 0 : result.status) === false)
            throw result.error ? result.error : 'SERVER_ERROR_CAN_NOT_SAVE_BATCH';
        if (evaluationPlan.evaluationStarted)
            return StartEvaluationPlan(vcc, evaluationPlan);
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_COULD_NOT_START_EVALUATION_PLAN';
        logger.info('[EvaluationPlanService::SaveEvaluationPlanBatch] Batch was saved.');
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error('[EvaluationPlanService::SaveEvaluationPlanBatch] Error: ' + errorMsg);
        CancelFailedBillingBatch(vcc, evaluationPlan);
        return Promise.resolve(false);
    });
}
function CreateScheduleForEvaluationPlan(vcc, vccType, evaluationPlan, process, hasAutomaticPart, params, userId) {
    var _a;
    var hasCampaigns = ((_a = params === null || params === void 0 ? void 0 : params.campaigns) === null || _a === void 0 ? void 0 : _a.length) ? params.campaigns.length : 0;
    var newId = (0, utilities_1.guid)();
    evaluationPlan.scheduleData.batchType = 'EVALUATION_PLAN';
    evaluationPlan.scheduleData.id = evaluationPlan.description;
    evaluationPlan.scheduleData.processFiltersId = newId;
    process !== null && process !== void 0 ? process : (process = {});
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (!hasCampaigns) {
            logger.info("[EvaluationPlanService::CreateScheduleForEvaluationPlan(".concat(evaluationPlan.id, ")] Searching evaluation format's campaigns"));
            return mssql_service_1.MSSql.GetCampaignsEvaluationFormat(vcc, params.evaluationFormatId);
        }
        else
            return Promise.resolve(true);
    })
        .then(function (result) {
        if (['allegro', 'i6', 'i-ftp', 'i-sftp'].includes(params.environmentType)) {
            if (!hasCampaigns) {
                if (!result || !result.length)
                    throw 'SERVER_ERROR_EVALUATION_FORMAT_CAMPAIGNS_NOT_FOUND';
                logger.info("[EvaluationPlanService::CreateScheduleForEvaluationPlan(".concat(evaluationPlan.id, ")] Campaigns on evaluationFormat: ").concat(result.length));
                params.campaigns = result.map(function (r) { return r.id; });
            }
            process.name = newId;
            process.processFilters = (0, interaction_service_1.GetProcessFilterFromInteractionSearch)(process.name, params, userId);
        }
        return Promise.resolve(true);
    })
        .then(function (_) {
        return (0, batch_schedule_service_1.SaveEvaluationPlanBatchSchedule)(vcc, vccType, evaluationPlan, process, userId);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error("[EvaluationPlanService::CreateScheduleForEvaluationPlan(".concat(evaluationPlan.id, ")] Error on:"), err);
        return Promise.resolve(false);
    });
}
function StartEvaluationPlan(vcc, evaluationPlan) {
    logger.info("[EvaluationPlanService::StartEvaluationPlan(".concat(evaluationPlan.id, ")] Starting evaluation plan."));
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (evaluationPlan.filterByContentQuery
            &&
                evaluationPlan.filterByContentQueryName) {
            logger.info("[EvaluationPlanService::StartEvaluationPlan(".concat(evaluationPlan.id, ")] Adding into evaluationPlan queue."));
            return CreateQueueL(vcc, evaluationPlan.processId, 'evaluationPlan', evaluationPlan.id);
        }
        else
            return Promise.resolve(true);
    })
        .then(function (_) {
        var promiseArray = [];
        if (['A', 'X'].includes(evaluationPlan.evaluation_mode)) {
            logger.info("[EvaluationPlanService::StartEvaluationPlan(".concat(evaluationPlan.id, ")] Adding into process queue"));
            promiseArray.push(CreateQueueL(vcc, evaluationPlan.processId, 'ability', evaluationPlan.id));
            promiseArray.push(CreateQueueL(vcc, evaluationPlan.processId, 'tipification', evaluationPlan.id));
        }
        return Promise.all(promiseArray);
    })
        .then(function (_) {
        logger.info("[EvaluationPlanService::StartEvaluationPlan(".concat(evaluationPlan.id, ")] Starting process"));
        return (0, linguistic_service_1.BeginProcess)(vcc, evaluationPlan.processId);
    })
        .then(function (result) {
        return Promise.resolve(true);
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Requesting error');
        logger.error("[EvaluationPlanService::StartEvaluationPlan(".concat(evaluationPlan.id, ")] Error on:"), err);
        return Promise.resolve(false);
    });
}
exports.StartEvaluationPlan = StartEvaluationPlan;
function GetEvaluationPlanModels(vcc, templateId) {
    return (0, promises_shared_1.DummyPromise)()
        .then(function (result) {
        return mssql_service_1.MSSql.GetTemplateThemeModels(vcc, templateId, null);
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_TEMPLATE_NLU_ITEMS_NOT_FOUND';
        var modelIntentIds = result.map(function (mi) { return mi.id; });
        return mssql_service_1.MSSql.GetUniqueNLUByIntents(vcc, templateId, modelIntentIds.join(','));
    }).then(function (result) {
        if (!result)
            throw 'SERVER_ERROR_NLUS_NOT_FOUND';
        return Promise.resolve(result);
    }).catch(function (err) {
        return Promise.reject(err);
    });
}
function SaveEvaluationPlan(vcc, evaluationPlan) {
    return mssql_service_1.MSSql.SaveEvaluationPlan(vcc, evaluationPlan.id, evaluationPlan.modality, moment(evaluationPlan.dateFin).utc().toDate(), evaluationPlan.createdDate, evaluationPlan.createdByUserId, evaluationPlan.lastModifiedDate, evaluationPlan.lastModifiedByUserId, evaluationPlan.idEvaluationFormat, evaluationPlan.transcriptionLanguage, evaluationPlan.environmentId, evaluationPlan.description, evaluationPlan.enabled, evaluationPlan.filterByContent, evaluationPlan.filterByContentPercentage, evaluationPlan.percentage, evaluationPlan.processAndEvaluate, evaluationPlan.filterByContentQueryName, evaluationPlan.filterByContentQuery, evaluationPlan.evaluationStarted, evaluationPlan.processId, evaluationPlan.status, evaluationPlan.evaluation_mode, evaluationPlan.batchScheduleId);
}
function SetAuditorInteractionsNumbersFromPercentage(auditors, totalInteractions) {
    var initQuantity = totalInteractions;
    var toFix = totalInteractions;
    if (toFix > 0) {
        auditors.forEach(function (a) {
            var cantidad = Math.round(a.percentage * initQuantity / 100);
            toFix -= cantidad;
            a.numberInteractions = cantidad;
        });
    }
    if (toFix !== 0) {
        if (toFix > 0) {
            for (var i = auditors.length - 1; i >= 0; i--) {
                if (auditors[i].selected) {
                    auditors[i].numberInteractions += toFix;
                    toFix = 0;
                    break;
                }
            }
        }
        else {
            for (var i = auditors.length - 1; i >= 0; i--) {
                if (toFix !== 0) {
                    for (var j = toFix; j < 0; j++) {
                        if (auditors[i].numberInteractions >= 1) {
                            auditors[i].numberInteractions--;
                            toFix++;
                        }
                        else
                            break;
                    }
                }
                else
                    break;
            }
        }
    }
    return auditors;
}
exports.SetAuditorInteractionsNumbersFromPercentage = SetAuditorInteractionsNumbersFromPercentage;
function SaveAuditorEP(vcc, evaluationPlan) {
    var promiseResult = true;
    return (0, promises_shared_1.DummyPromise)()
        .then(function (_) {
        if (evaluationPlan.auditors)
            return mssql_service_1.MSSql.BulkAuditorEvaluationPlan(vcc, evaluationPlan);
        else
            return Promise.resolve([]);
    })
        .then(function (result) {
        logger.info("[EvaluationPlanService::SaveAuditorEP] Save evaluations: ".concat(result));
        return Promise.resolve(result);
    })
        .catch(function (err) {
        promiseResult = false;
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
function CreateQueueL(vcc, processId, typeName, typeId) {
    var currentQueueId = '';
    currentQueueId = vcc;
    currentQueueId += ':';
    currentQueueId += processId;
    if (typeName && typeId) {
        currentQueueId += ':';
        currentQueueId += typeName;
        currentQueueId += ':';
        currentQueueId += typeId;
    }
    return redis_service_1.Redis.CreateQueueL('Process', currentQueueId)
        .then(function (result) {
        if (result)
            return true;
        else
            return false;
    })
        .catch(function (err) {
        var clientMsg = (typeof err === 'string' ? err : 'CANNOT_CREATE_QUEUE');
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
        logger.error('[ProcessService::CreateQueue] getting errorMsg:', errorMsg);
    });
}
exports.CreateQueueL = CreateQueueL;
function RemoveEvaluationPlanProcess(vcc, evaluationPlan) {
    var myProcessId = evaluationPlan.processId;
    var promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(function (result) {
        return mssql_service_1.MSSql.DeleteProcess(vcc, myProcessId);
    })
        .then(function (result) {
        if (!result)
            throw 'ERROR_DELETING_PROCESS';
        return Promise.resolve(promiseResult);
    })
        .catch(function (err) {
        promiseResult = false;
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::SaveAuditorEP] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.RemoveEvaluationPlanProcess = RemoveEvaluationPlanProcess;
function CleanFailedEvaluationPlanData(vcc, evaluationPlan) {
    var myEvaluationPlanId = evaluationPlan.id;
    var promiseResult = true;
    return Promise.resolve(promiseResult)
        .then(function (result) {
        return mssql_service_1.MSSql.CleanFailedEvaluationPlanData(vcc, myEvaluationPlanId);
    })
        .then(function (result) {
        if (!result)
            throw 'ERROR_CLEANING_FAILED_EVALUATION_PLAN_DATA';
        return Promise.resolve(promiseResult);
    })
        .catch(function (err) {
        promiseResult = false;
        var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'Executing request error');
        logger.error('[EvaluationPlanService::CleanFailedEvaluationPlanData] Error: ' + errorMsg);
        return Promise.reject(promiseResult);
    });
}
exports.CleanFailedEvaluationPlanData = CleanFailedEvaluationPlanData;
function CancelFailedBillingBatch(vcc, evaluationPlan) {
    var promiseResult = true;
    var batchManage = new billing_batch_model_1.BatchManage;
    batchManage.vcc = vcc;
    batchManage.batchTypeName = 'EVALUATION_PLAN';
    batchManage.batchTypeId = evaluationPlan.id;
    batchManage.systemService = 'ALL';
    batchManage.user = evaluationPlan.lastModifiedByUserId;
    return Promise.resolve(promiseResult)
        .then(function (result) {
        var billingBatch = new billing_batch_implementation_1.BillingBatchImplementation();
        return billingBatch.CancelBatchSummary(batchManage);
    })
        .catch(function (err) {
        promiseResult = false;
        var batchRunningStatus = null;
        mssql_service_1.MSSql.SaveBatchStatus(vcc, batchManage.batchTypeId, batchManage.systemService, batchRunningStatus);
        logger.error('[EvaluationPlanService::create_evaluationplan_error] Error on \'' + err.toString());
        return Promise.reject(promiseResult);
    });
}
exports.CancelFailedBillingBatch = CancelFailedBillingBatch;
