"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WorkGroupSupervisorService = void 0;
var moment = require("moment");
var log4js = require("log4js");
var mssql_service_1 = require("../db/mssql.service");
var promises_shared_1 = require("../shared/promises.shared");
var logger = log4js.getLogger('ServerScripts');
function WorkGroupSupervisorService(app) {
    app.post('/api/workgroupssupervisor/get_workgroupssupervisor', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var workGroupSupervisor = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetWorkGroupsSupervisor(vcc, workGroupSupervisor.searchCriteria, workGroupSupervisor.searchExpression);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
    app.post('/api/workgroupsupervisor/create_workgroupsupervisor', function (req, res, next) {
        logger.info('[WorkGroupSupervisor::create_workgroupsupervisor] Received ' + req.method + ' method on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var userId = res.locals.userId;
        var nowUtc = moment().utc().toDate();
        var workGroupSupervisorCollection = (req.body || {});
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            var promiseArray = [];
            workGroupSupervisorCollection.forEach(function (wgsc) {
                promiseArray.push(mssql_service_1.MSSql.SaveWorkGroupSupervisor(vcc, userId, nowUtc, wgsc));
            });
            return Promise.all(promiseArray);
        })
            .then(function (resultSave) {
            if (res.headersSent)
                return;
            res.json({ status: resultSave, description: (resultSave ? 'SERVER_SUCCESS_MESSAGE' : 'ERROR_EFECTIVITY_ID_ALREADY_EXISTS'), data: resultSave });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('[WorkGroupSupervisor::create_workgroupsupervisor] Error on \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg });
        });
    });
    app.get('/api/workgroupsupervisor/get_workgroupsupervisor/:supervisor_id', function (req, res, next) {
        logger.info('[WorkGroupSupervisorService::get_workgroupsupervisor] Received ' + req.method + ' on \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        logger.info('[WorkGroupSupervisor::get_workgroupsupervisor] Getting vcc:', vcc);
        var supervisorId = req.params.supervisor_id;
        logger.info('[WorkGroupSupervisor::get_workgroupsupervisor] Getting supervisorId:', supervisorId);
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            if (!supervisorId)
                throw 'MISSING_SUPERVISOR_INPUT_PARAMTER';
            return mssql_service_1.MSSql.GetWorkGroupSupervisor(vcc, supervisorId);
        })
            .then(function (resultWorkGroupSupervisor) {
            logger.info('[ProcessService::get_workgroupsupervisor] Getting resultProcess after MSSql.GetWorkGroupSupervisor');
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: resultWorkGroupSupervisor });
        })
            .catch(function (err) {
            var processMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            logger.error('[ProcessService::get_workgroupsupervisor] Getting errorMsg:', errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: processMsg, error: errorMsg });
        });
    });
    app.post('/api/workgroupsupervisor/get_workgroupsfromsupervisorEP', function (req, res, next) {
        logger.info('Se recibio ' + req.method + ' para \'' + req.path + '\'');
        var vcc = res.locals.vcc;
        var campaignIds = (req.body || {});
        var campaignIdsString = campaignIds.join(',');
        (0, promises_shared_1.DummyPromise)()
            .then(function (result) {
            return mssql_service_1.MSSql.GetWorkGroupsFromSupervisorEP(vcc, campaignIds);
        }).then(function (result) {
            if (res.headersSent)
                return;
            res.json({ status: true, description: 'SERVER_SUCCESS_MESSAGE', data: result });
        }).catch(function (err) {
            var clientMsg = (typeof err === 'string' ? err : 'CANNOT_PROCESS_REQUEST');
            var errorMsg = (typeof err === 'string' ? err : err.message || err.description || 'SERVER_ERROR_UNKNOWN_MESSAGE');
            logger.error('Error al procesar \'' + req.path + '\': ' + errorMsg);
            if (res.headersSent)
                return;
            res.json({ status: false, description: clientMsg, error: errorMsg, detail: { type: 'ERROR', datetime: moment().utc().toDate(), pid: process.pid, source: __filename, stack: err.stack } });
        });
    });
}
exports.WorkGroupSupervisorService = WorkGroupSupervisorService;
